@@ -1,17 +1,15 @@
-"""f90nml.findex
-   =============
+"""Column-major Fortran iterator of indices across multipe dimensions.
 
-   Column-major Fortran iterator of indices across multipe dimensions.
-
-   :copyright: Copyright 2015 Marshall Ward, see AUTHORS for details.
-   :license: Apache License, Version 2.0, see LICENSE for details.
+:copyright: Copyright 2015 Marshall Ward, see AUTHORS for details.
+:license: Apache License, Version 2.0, see LICENSE for details.
 """
 
 
 class FIndex(object):
-    """Column-major multidimensional index iterator"""
+    """Column-major multidimensional index iterator."""
 
     def __init__(self, bounds, first=None):
+        """Initialise the index iterator."""
         self.start = [1 if b[0] is None else b[0] for b in bounds]
         self.end = [b[1] for b in bounds]
         self.step = [1 if b[2] is None else b[2] for b in bounds]
@@ -25,12 +23,15 @@ class FIndex(object):
             self.first = [b[0] for b in bounds]
 
     def __iter__(self):
+        """Declare object as iterator."""
         return self
 
     def next(self):
+        """Python 2 interface to Python 3 iterator."""
         return self.__next__()
 
     def __next__(self):
+        """Iterate to next contiguous index tuple."""
         if self.end[-1] and self.current[-1] >= self.end[-1]:
             raise StopIteration
 
