@@ -7,8 +7,9 @@ a Python environment.
 :license: Apache License, Version 2.0, see LICENSE for details.
 """
 from __future__ import print_function
-import itertools
+
 import copy
+import itertools
 import numbers
 import os
 import platform
@@ -24,6 +25,32 @@ try:
     basestring          # Python 2.x
 except NameError:
     basestring = str    # Python 3.x
+try:
+    from collections.abc import KeysView    # Python 3.4+
+    from collections.abc import ItemsView    # Python 3.4+
+except ImportError:
+    from collections import KeysView        # Python 2.7 - 3.3
+    from collections import ItemsView        # Python 2.7 - 3.3
+
+
+class _NamelistKeysView(KeysView):
+    """Return the namelist's KeysView based on the Namelist iterator."""
+
+    def __iter__(self):
+        # This is faster, but requires Python 3.x
+        # yield from self._mapping
+
+        # This is slower but works in 2.7
+        for key in self._mapping:
+            yield key
+
+
+class _NamelistItemsView(ItemsView):
+    """Return the namelist's ItemsView based on the Namelist iterator."""
+
+    def __iter__(self):
+        for key in self._mapping:
+            yield (key, self._mapping[key])
 
 
 class Namelist(OrderedDict):
@@ -50,18 +77,33 @@ class Namelist(OrderedDict):
         s_args = list(args)
 
         # If using (unordered) dict, then resort the keys for reproducibility
+        # NOTE: Python 3.7+ dicts are order-preserving.
+        # FIXME: I need to drop this, no longer relevant.
         if (args and not isinstance(args[0], OrderedDict) and
                 isinstance(args[0], dict)):
             s_args[0] = sorted(args[0].items())
 
-        # Assign the default start index
-        try:
-            self._default_start_index = kwds.pop('default_start_index')
-        except KeyError:
-            self._default_start_index = None
+        # Passing a Namelist through Namelist() will convert cogroup keyed
+        #   values to lists.  To prevent this, we convert back to dicts.
+        if args and isinstance(args[0], Namelist):
+            s_args[0] = s_args[0].todict()
 
+        # Assign the default start index
+        self._default_start_index = kwds.pop('default_start_index', None)
+
+        # Initialize via OrderedDict
         super(Namelist, self).__init__(*s_args, **kwds)
 
+        # Construct the cogroups based on the internal key values
+        self._cogroups = {}
+        for key in self:
+            if key.startswith('_grp_'):
+                grp = key[5:].rsplit('_', 1)[0]
+                try:
+                    self._cogroups[grp].append(key)
+                except KeyError:
+                    self._cogroups[grp] = [key]
+
         self.start_index = self.pop('_start_index', {})
 
         # Update the complex tuples as intrinsics
@@ -105,27 +147,71 @@ class Namelist(OrderedDict):
 
     def __delitem__(self, key):
         """Case-insensitive interface to OrderedDict."""
-        return super(Namelist, self).__delitem__(key.lower())
+        lkey = key.lower()
+
+        if isinstance(key, NmlKey):
+            super(Namelist, self).__delitem__(key._key)
+        elif lkey in self._cogroups:
+            # Remove all cogroup values
+            cogrp = Cogroup(self, lkey)
+            for gkey in cogrp.keys:
+                super(Namelist, self).__delitem__(gkey)
+
+            self._cogroups.remove(lkey)
+        else:
+            super(Namelist, self).__delitem__(key)
 
     def __getitem__(self, key):
         """Case-insensitive interface to OrderedDict."""
-        if isinstance(key, basestring):
-            return super(Namelist, self).__getitem__(key.lower())
+        if isinstance(key, NmlKey):
+            return super(Namelist, self).__getitem__(key._key)
+        elif isinstance(key, basestring):
+            lkey = key.lower()
+
+            if lkey in self._cogroups:
+                return Cogroup(self, lkey)
+            else:
+                return super(Namelist, self).__getitem__(lkey)
         else:
             keyiter = iter(key)
             grp, var = next(keyiter).lower(), next(keyiter).lower()
             return super(Namelist, self).__getitem__(grp).__getitem__(var)
 
+    def __iter__(self):
+        """Implement iter(self)."""
+        for key in super(Namelist, self).__iter__():
+            yield NmlKey(key)
+
+    # NOTE: Since we override __getitem__, we must also override get()
+    def get(self, key, default=None):
+        """Return the value for key if key is in the Namelist, else default."""
+        try:
+            return self[key]
+        except KeyError:
+            return default
+
+    def keys(self):
+        """Return the namelist keys as a KeysView."""
+        return _NamelistKeysView(self)
+
+    def items(self):
+        """Return the namelist keys as an ItemsView."""
+        return _NamelistItemsView(self)
+
     def __setitem__(self, key, value):
         """Case-insensitive interface to OrderedDict.
 
         Python dict inputs to the Namelist, such as derived types, are also
         converted into Namelists.
         """
+        # Promote dicts to Namelists
         if isinstance(value, dict) and not isinstance(value, Namelist):
-            value = Namelist(value,
-                             default_start_index=self.default_start_index)
+            value = Namelist(
+                value,
+                default_start_index=self.default_start_index
+            )
 
+        # Convert list of dicts to list of namelists
         elif is_nullable_list(value, dict):
             for i, v in enumerate(value):
                 if isinstance(v, Namelist) or v is None:
@@ -137,7 +223,11 @@ class Namelist(OrderedDict):
                         default_start_index=self.default_start_index
                     )
 
-        super(Namelist, self).__setitem__(key.lower(), value)
+        if isinstance(key, NmlKey):
+            super(Namelist, self).__setitem__(key._key, value)
+        else:
+            lkey = key.lower()
+            super(Namelist, self).__setitem__(lkey, value)
 
     def __str__(self):
         """Print the Fortran representation of the namelist.
@@ -487,6 +577,66 @@ class Namelist(OrderedDict):
             if not nml_is_file:
                 nml_file.close()
 
+    def create_cogroup(self, group_name):
+        """Convert an existing namelist group to a cogroup."""
+        grp_key = group_name.lower()
+
+        if grp_key in self._cogroups:
+            return
+
+        if grp_key in self:
+            # Move existing group to an internal cogroup key
+
+            # NOTE: Since OrderedDict does not permit a renaming of keys, we
+            # are forced to remove and replace the entry with the new internal
+            # key - which moves it to the end - and then remove/reappend all
+            # keys after this key.
+
+            nml_groups = list(self.keys())
+            grp_idx = nml_groups.index(grp_key)
+
+            # Remove the existing value and add to the end.
+            grp_val = self.pop(grp_key)
+
+            cogrp_key = ''.join(['_grp_', grp_key, '_0'])
+            self[cogrp_key] = grp_val
+
+            # Remove and replace existing keys after the new cogroup key.
+            for key in nml_groups[grp_idx+1:]:
+                grp = self.pop(key)
+                self[key] = grp
+
+            cogroup_keys = [cogrp_key]
+        else:
+            cogroup_keys = []
+
+        self._cogroups[grp_key] = cogroup_keys
+
+    def add_cogroup(self, key, val):
+        """Append a duplicate group to the Namelist as a new group."""
+        lkey = key.lower()
+
+        if lkey in self and lkey not in self._cogroups:
+            self.create_cogroup(lkey)
+
+        # Generate the cogroup label and add to the Namelist
+        # NOTE: In order to preserve ordering, we cannot reuse a key which may
+        # have been removed.  So we always generate a new key based on the
+        # largest index.
+
+        # Gather the list of existing IDs
+        hdr = '_grp_{0}_'.format(lkey)
+        idx = [int(k.split(hdr)[1]) for k in self._cogroups[lkey]]
+        try:
+            cogrp_id = 1 + max(idx)
+        except ValueError:
+            # If an empty cogroup was set up, use '0' for the label.
+            cogrp_id = 0
+
+        cogrp_key = '_'.join(['_grp', lkey, str(cogrp_id)])
+        self[cogrp_key] = val
+        self._cogroups[lkey].append(cogrp_key)
+
     def patch(self, nml_patch):
         """Update the namelist from another partial or full namelist.
 
@@ -534,6 +684,10 @@ class Namelist(OrderedDict):
             print(file=nml_file)
         self._newline = True
 
+        # Strip metadata label for repeat groups
+        if grp_name.startswith('_grp_'):
+            grp_name = grp_name[5:].rsplit('_', 1)[0]
+
         if self.uppercase:
             grp_name = grp_name.upper()
 
@@ -752,12 +906,16 @@ class Namelist(OrderedDict):
         namelist into an equivalent format which does not support complex
         numbers, such as JSON or YAML.
         """
-        # TODO: Preserve ordering
-        nmldict = OrderedDict(self)
+        # Copy Namelist to OrderedDict, converting NmlKeys back to strings
+        nmldict = OrderedDict()
+        for key, value in self.items():
+            nmldict[str(key)] = value
 
         # Search for namelists within the namelist
         # TODO: Move repeated stuff to new functions
-        for key, value in self.items():
+        for nkey, value in self.items():
+            key = str(nkey)
+
             if isinstance(value, Namelist):
                 nml = copy.deepcopy(value)
                 nmldict[key] = nml.todict(complex_tuple)
@@ -848,6 +1006,80 @@ class Namelist(OrderedDict):
         return result
 
 
+class Cogroup(list):
+    """List of Namelist groups which share a common key.
+
+    Although Namelists are organized as associative arrays, access is
+    typically through a serial I/O data stream.  One consequence is that a
+    namelist may contain multiple keys for different values.
+
+    This object returns a list of namelist groups which use the same key.
+    Internal keys correspond to the original ordering in the namelist.
+
+    When an element of the list is updated, the corresponding namelist element
+    is also updated.
+    """
+
+    def __init__(self, nml, key, *args, **kwds):
+        """Generate list of Namelist cogroups linked to parent namelist."""
+        self.nml = nml
+        self.key = key
+
+        grps = [self.nml[k] for k in self.keys]
+        super(Cogroup, self).__init__(grps, **kwds)
+
+    def __setitem__(self, index, value):
+        """Set the namelist key at position index in the Cogroup to value.
+
+        The Cogroup list contains references to its associated namelist.  When
+        this Cogroup element value is assigned, it is also set in the namelist.
+        """
+        key = self.keys[index]
+        self.nml[key] = value
+
+    def __delitem__(self, index):
+        """Delete the namelist key at position index in the Cogroup.
+
+        The Cogroup list contains references to its associated namelist.  When
+        an element is deleted from the Cogroup, it is passed to the namelist
+        and deleted.
+        """
+        key = self.keys[index]
+        del self.nml[key]
+        super(Cogroup, self).__delitem__(index)
+
+        # Remove the cogroup status if keys are depleted
+        if len(self) == 0:
+            self.nml._cogroups.remove(self.key)
+
+    # NOTE: Maybe this function is no longer useful...
+    @property
+    def keys(self):
+        """Return the namelist keys in the cogroup."""
+        return [k for k in self.nml if k == self.key]
+
+
+class NmlKey(str):
+    """Create a new NmlKey object.
+
+    An NmlKey is a str containing an internal key used to distinguish between
+    duplicate keys in the namelist.
+    """
+
+    def __new__(cls, value='', *args, **kwargs):
+        """Create and return a new NmlKey."""
+        name = _cogroup_basename(value)
+        tok = str.__new__(cls, name, *args)
+        tok._key = value
+
+        return tok
+
+
+def _cogroup_basename(grp):
+    """Return the cogroup name from the internal key."""
+    return grp[5:].rsplit('_', 1)[0] if grp.startswith('_grp_') else grp
+
+
 def is_nullable_list(val, vtype):
     """Return True if list contains either values of type `vtype` or None."""
     return (isinstance(val, list) and
