@@ -396,6 +396,15 @@ class Parser(object):
 
                     g_vars[v_name] = v_values
 
+                    # Squeeze 1d list due to repeated variables
+                    for v_name, v_values in g_vars.items():
+                        if (
+                            isinstance(v_values, list)
+                            and len(v_values) == 1
+                            and v_name not in g_vars.start_index
+                        ):
+                            g_vars[v_name] = v_values[0]
+
                     # Deselect variable
                     v_name = None
                     v_values = []
@@ -412,18 +421,9 @@ class Parser(object):
 
                     # Append the grouplist to the namelist
                     if g_name in nmls:
-                        g_update = nmls[g_name]
-
-                        # Update to list of groups
-                        if not isinstance(g_update, list):
-                            g_update = [g_update]
-
-                        g_update.append(g_vars)
-
+                        nmls.add_cogroup(g_name, g_vars)
                     else:
-                        g_update = g_vars
-
-                    nmls[g_name] = g_update
+                        nmls[g_name] = g_vars
 
                     # Reset state
                     g_name, g_vars = None, None
@@ -640,12 +640,11 @@ class Parser(object):
                     #       rather than prepending it to the iterator.
                     self.tokens, pre_lookahead = itertools.tee(self.tokens)
                     lookahead = itertools.chain([self.token], pre_lookahead)
-                    n_vals_remain = count_values(lookahead)
 
                     if patch_values:
                         # TODO: Patch indices that are not set in the namelist
                         if (p_idx < len(patch_values)
-                                and n_vals_remain > 0
+                                and check_for_value(lookahead)
                                 and self.token != ','):
                             p_val = patch_values[p_idx]
                             p_repr = patch_nml._f90repr(patch_values[p_idx])
@@ -967,17 +966,19 @@ def delist(values):
     return values
 
 
-def count_values(tokens):
-    """Identify the number of values ahead of the current token."""
+def check_for_value(tokens):
+    """Return True if the next token is a value to be assigned."""
     ntoks = 0
     for tok in tokens:
         if tok.isspace() or tok == ',':
             continue
         elif tok in ('=', '/', '$', '&'):
-            if ntoks > 0 and tok == '=':
-                ntoks -= 1
             break
         else:
             ntoks += 1
 
-    return ntoks
+        # If ntoks reaches 2, then there must be at least one value.
+        if ntoks > 1:
+            break
+
+    return ntoks > 0
